/***************************************************************************
 *   Copyright (C) 2005 by Andreas Silberstorff   *
 *   ml@andreas-silberstorff.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/

#include "kalva_recorder.h"
#include "settings.h"
#include "prefs.h"

#include <cmath>

#include <qcheckbox.h>
#include <qdir.h>
#include <qstring.h>
#include <qlineedit.h>
#include <qtabwidget.h>

#include <kdebug.h>
#include <klocale.h>
#include <kprocess.h>
#include <ktimewidget.h>
#include <kdatepicker.h>
#include <kcombobox.h>
#include <kinputdialog.h>
#include <kapplication.h>
#include <kstandarddirs.h>
#include <kuser.h>

#include <kmessagebox.h>

#include <kdeversion.h>
#undef KDE_3_3_FEATURES
#if defined(KDE_MAKE_VERSION)
#if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
	#define KDE_3_3_FEATURES
#endif
#endif

KalvaRecorder::KalvaRecorder(QWidget* parent, const char* name, WFlags fl)
    : kalva_recordergui(parent,name,fl)
{
   connect( m_film, SIGNAL( activated( const QString& )),
            m_film, SLOT( addToHistory( const QString& )));
   m_film->setMaxCount(100);
   m_film->setDuplicatesEnabled (false);
   m_user = 0;
   m_proc = 0;
   m_dcopProc = 0;
}

KalvaRecorder::~KalvaRecorder()
{}

/*$SPECIALIZATION$*/
void KalvaRecorder::setWeekends()
{
   m_monday->setChecked(false);
   m_tuesday->setChecked(false);
   m_wednesday->setChecked(false);
   m_thursday->setChecked(false);
   m_friday->setChecked(false);
   m_saturday->setChecked(true);
   m_sunday->setChecked(true);
}

void KalvaRecorder::setLaborDays()
{
   m_monday->setChecked(true);
   m_tuesday->setChecked(true);
   m_wednesday->setChecked(true);
   m_thursday->setChecked(true);
   m_friday->setChecked(true);
   m_saturday->setChecked(false);
   m_sunday->setChecked(false);
}

void KalvaRecorder::setDaily()
{
   m_monday->setChecked(true);
   m_tuesday->setChecked(true);
   m_wednesday->setChecked(true);
   m_thursday->setChecked(true);
   m_friday->setChecked(true);
   m_saturday->setChecked(true);
   m_sunday->setChecked(true);
}

void KalvaRecorder::stationChanged(const QString stat, const QString chan, const QString freq)
{
   m_station->setText(stat );
   m_channel->setText(chan );
   m_frq->setText(freq );
}


/* main control */

void KalvaRecorder::record()
{
   int recordMode = m_recordMode->currentPageIndex();
   switch (recordMode)
   {
      case 0:
         slotRecordNow();
         return;
      case 1:
         slotRecordMovie();
         return;
      case 2:
         slotRecordSerial();
         return;
   }
   cmd.append("***\nrecordMode = ");
   cmd.append(recordMode);
   KMessageBox::error ( this, cmd, i18n ("unsupported recordmode") );
}

void KalvaRecorder::slotDelete()
{
   int recordMode = m_recordMode->currentPageIndex();
   switch (recordMode)
   {
      case 0:
         return;
      case 1:
         if (! m_jobId->text().isEmpty() )  {
            emit signalDeleteMovie( m_jobId->text(), true );
         }
         return;
      case 2:
         emit signalDeleteSerialFromCron();
         return;
   }
}


/**
 * Record Now
 */
void KalvaRecorder::slotRecordNow()
{
    kdDebug() << "in slotRecordNow " << endl;
    kdDebug() << "videodev = " << Settings::videodev() << endl;
    ///@TODO this is a HACK, make it better!
    if ( Settings::driver() == i18n( "DVB (experimental!)" ) ) {
       cmd   = recordCmdDVB();
    }  else  {
       cmd   = recordCmd();
    }

    if ( Settings::show_command() == true  ) {
#ifdef KDE_3_3_FEATURES
       bool ok;
       cmd = KInputDialog::getMultiLineText (
          i18n("Command to run"), 
          i18n("Command"), 
          cmd, 
          &ok,
          this,
          "cmdDlg"
       );
       if ( ok != true )  {
          return;
       }
#else
       int ret = KMessageBox::warningContinueCancel (
          this,
          cmd, 
          i18n("Command to run")
       );
       if ( ret != KMessageBox::Continue ) {
          return;
       }
#endif
   }

   if (m_proc)  {
      delete m_proc;
      m_proc = 0;
   }
   m_proc = new KProcess;
   QApplication::connect(m_proc, SIGNAL(processExited(KProcess *)),
                         this,   SLOT(slotMencRecNowExited(KProcess *)));
   m_proc->clearArguments();
   m_proc->setUseShell(true);
   (*m_proc) << cmd;
   kdDebug() << "Calling " << cmd << endl;
   m_proc->start(KProcess::NotifyOnExit, KProcess::Stderr);
}

void KalvaRecorder::slotMencRecNowExited(KProcess * proc)
{
   kdDebug() << "in slotMencRecNowExited(KProcess * proc)" << endl;
   if (! proc->normalExit() ) {
      KMessageBox::detailedSorry   (
         0,
         i18n("mencoder exited with errors."), 
         i18n("mencoder output was:\n%1").arg( proc->exitStatus() ), 
         i18n("mencoder failure")
      );
      kdWarning() << "mencoder exited with status"
                  << proc->exitStatus() << endl;
      kdWarning() << "mencoder exited abnormal" << endl;
      return;
   }
   //emit signalVideoRecorded();
}

/* Serial Recording */

void KalvaRecorder::slotRecordMovie()
{
    ///@TODO this is a HACK, make it better!
    if ( Settings::driver() == i18n( "DVB (experimental!)" ) ) {
       cmd   = recordCmdDVB();
    }  else  {
       cmd   = recordCmd();
    }

    if ( cmd.isEmpty() )  {
       return;
    }
    cmd.replace(QChar('"'), "\\\"");
    cmd.replace(QChar('`'), "\\`");
    cmd.prepend("echo \"");
    cmd.append("\" | at " + m_startMovie->time().toString ("hh:mm") + " " +
                            m_calender->date().toString("dd.MM.yyyy") );

    if ( Settings::show_command() == true  ) {
#ifdef KDE_3_3_FEATURES
       bool ok;
       cmd = KInputDialog::getMultiLineText (
          i18n("Command to run"), 
          i18n("Command"), 
          cmd, 
          &ok,
          this,
          "cmdDlg"
       );
       if ( ok != true )  {
          return;
       }
#else
       int ret = KMessageBox::warningContinueCancel (
          this,
          cmd, 
          i18n("Command to run")
       );
       if ( ret != KMessageBox::Continue ) {
          return;
       }
#endif
   }
   if (m_proc)  {
      delete m_proc;
      m_proc = 0;
   }
   m_proc = new KProcess;
   QApplication::connect(m_proc, SIGNAL(processExited(KProcess *)),
                         this,   SLOT(slotMencRecMovieExited(KProcess *)));
   m_proc->clearArguments();
   m_proc->setUseShell(true);
   (*m_proc) << cmd;
   kdDebug() << "Calling " << cmd << endl;
   m_proc->start(KProcess::NotifyOnExit, KProcess::Stderr);

   if ( m_dcopProc )
     delete m_dcopProc;
   m_dcopProc = 0;
   m_dcopProc = new KProcess;
   m_dcopProc->clearArguments();
   m_dcopProc->setUseShell(true);

   QTime dcoptime(m_startMovie->time());
   dcoptime = dcoptime.addSecs(180);
   cmd = "echo \"";
   cmd.append( "dcop --user $USER" );
   cmd.append( " --all-sessions" ); 
   cmd.append( " kalva DCOP_kalva DcopMovieManagerRead" );
   cmd.append( "\" | at " + dcoptime.toString ("hh:mm") + " " +
                           m_calender->date().toString("dd.MM.yyyy") );
   (*m_dcopProc) << cmd;
   kdDebug() << "Calling " << cmd << endl;
   m_dcopProc->start(KProcess::DontCare, KProcess::Stderr);
}

void KalvaRecorder::slotMencRecMovieExited(KProcess * proc)
{
   kdDebug() << "in slotMencRecMovieExited(KProcess * proc)" << endl;
   if (! proc->normalExit() ) {
      KMessageBox::detailedSorry   (
         0,
         i18n("at-command exited with errors."), 
         i18n("cmd output was:\n%1").arg( proc->exitStatus() ), 
         i18n("at-command failure")
      );
      kdWarning() << "at exited with status"
                  << proc->exitStatus() << endl;
      kdWarning() << "at exited abnormal" << endl;
      return;
   }
   QString jobid = m_jobId->text();
   if ( jobid.isEmpty() )  {
      kdDebug() << "emiting signalMovieScheduled ()" << endl;
      emit signalMovieScheduled();
   }  else  {
      kdDebug() << "emiting signalDeleteMovie (" << jobid << ")" << endl;
      emit signalDeleteMovie( jobid, false );
   }
}

void KalvaRecorder::slotNewMovie()
{
    m_jobId->clear();
    m_startMovie->setTime(QTime::currentTime());
    m_calender->setDate(QDate::currentDate());
    m_film->clear();
    m_film->setFocus();
}

void KalvaRecorder::slotMoviesRead()
{
   slotNewMovie();
}


/* Serial Recording */

void KalvaRecorder::slotRecordSerial()
{
    ///@TODO this is a HACK, make it better!
    if ( Settings::driver() == i18n( "DVB (experimental!)" ) ) {
       cmd   = recordCmdDVB();
    }  else  {
       cmd   = recordCmd();
    }

    if ( cmd.isEmpty() )  {
       return;
    }
    cmd.replace(QChar('"'), "\\\"");
    cmd.replace(QChar('`'), "\\`");
    cmd.replace(QChar('%'), "\\%");
    cmd.prepend("perl /usr/bin/tvapp.pl -film \"" + m_film->currentText() + "\"" +
                                         " -hour "          +
                                         m_startSerial->time().toString("h") + 
                                         " -minute "        +
                                         m_startSerial->time().toString("m") + 
                                         " -daysoftheweek " + setDOWs() +
                                         " -cmd \""
               );
    cmd.append("\" -cronaddcmd ");

    if ( Settings::show_command() == true  ) {
#ifdef KDE_3_3_FEATURES
       bool ok;
       cmd = KInputDialog::getMultiLineText (
          i18n("Command to run"), 
          i18n("Command"), 
          cmd, 
          &ok,
          this,
          "cmdDlg"
       );
       if ( ok != true )  {
          return;
       }
#else
       int ret = KMessageBox::warningContinueCancel (
          this,
          cmd, 
          i18n("Command to run")
       );
       if ( ret != KMessageBox::Continue ) {
          return;
       }
#endif
   }

   if (m_proc)  {
      delete m_proc;
      m_proc = 0;
   }
   m_proc = new KProcess;
   QApplication::connect(m_proc, SIGNAL(processExited(KProcess *)),
                         this,   SLOT(slotMencRecSerialExited(KProcess *)));
   m_proc->clearArguments();
   m_proc->setUseShell(true);
   (*m_proc) << cmd;
   kdDebug() << "Calling " << cmd << endl;
   m_proc->start(KProcess::NotifyOnExit, KProcess::Stderr);
}

void KalvaRecorder::slotMencRecSerialExited(KProcess * proc)
{
   kdDebug() << "in slotMencRecSerialExited(KProcess * proc)" << endl;
   if (! proc->normalExit() ) {
      KMessageBox::detailedSorry   (
         0,
         i18n("tvapp.pl -cronaddcmd exited with errors."), 
         i18n("tvapp.pl output was:\n%1").arg( proc->exitStatus() ), 
         i18n("tvapp.pl failure")
      );
      kdWarning() << "tvapp.pl -cronaddcmd exited with status"
                  << proc->exitStatus() << endl;
      kdWarning() << "tvapp.pl exited abnormal" << endl;
      return;
   }
   emit signalSerialScheduled();
}

QString KalvaRecorder::setDOWs()  {
   dow = "";
   dow = setDayInDOWs ( m_sunday->isChecked(),    dow, ((QString)"0"));
   dow = setDayInDOWs ( m_monday->isChecked(),    dow, ((QString)"1"));
   dow = setDayInDOWs ( m_tuesday->isChecked(),   dow, ((QString)"2"));
   dow = setDayInDOWs ( m_wednesday->isChecked(), dow, ((QString)"3"));
   dow = setDayInDOWs ( m_thursday->isChecked(),  dow, ((QString)"4"));
   dow = setDayInDOWs ( m_friday->isChecked(),    dow, ((QString)"5"));
   dow = setDayInDOWs ( m_saturday->isChecked(),  dow, ((QString)"6"));
   ///@TODO: translation if there is a sequence in the list (f.i."1,2,3,4,5" should become "1-5")
   return dow;
}

QString KalvaRecorder::setDayInDOWs ( bool isChecked, QString dow, QString day )
{
   if (isChecked == true )  {
      if (! (dow.isEmpty() ))   {
          dow.append(",");
      }
      dow.append(day);
   }
   return dow;
}

///@TODO: translation if there is a sequence in the list (f.i."1,2,3,4,5" should become "1-5")
void KalvaRecorder::setYourDOWs ( const QString & yourDOWs )
{
   QStringList dows = QStringList::split( ",", yourDOWs, false );
   QString dow;
   bool ok;
   unsetDOWs();
   for ( QStringList::Iterator it = dows.begin(); it != dows.end(); ++it ) {
       dow = *it;
       kdDebug() << "dow = "  << dow << endl;
       setYourDayInDOWs ( dow.toInt( &ok, 10 ) );
   }
}

void KalvaRecorder::unsetDOWs ()
{
   m_sunday->setChecked(false);
   m_monday->setChecked(false);
   m_tuesday->setChecked(false);
   m_wednesday->setChecked(false);
   m_thursday->setChecked(false);
   m_friday->setChecked(false);
   m_saturday->setChecked(false);
}

void KalvaRecorder::setYourDayInDOWs ( const int & yourDOW )
{
   switch (yourDOW) {
      case 0: { m_sunday->setChecked(true);
                break; }
      case 1: { m_monday->setChecked(true);
                break; }
      case 2: { m_tuesday->setChecked(true);
                break; }
      case 3: { m_wednesday->setChecked(true);
                break; }
      case 4: { m_thursday->setChecked(true);
                break; }
      case 5: { m_friday->setChecked(true);
                break; }
      case 6: { m_saturday->setChecked(true);
                break; }
   }
}


// BUILD the mencoder call
QString KalvaRecorder::recordCmd()
{
    bool    useFrq   = Settings::use_frq();

    tv_scale     = Settings::tv_scale();
    x_scale      = tv_scale.section( ":", 0, 0 );
    y_scale      = tv_scale.section( ":", 1, 1 );
    y_scale      = y_scale.section( " ", 0, 0 );

    QString aumix     = Settings::aumix();
    QString aumixdev  = Settings::aumixdev();
    int nice_value    = Settings::nice_value();

    QString path      = Settings::storage_path();
    QString filmtitle = m_film->currentText();
    if ( filmtitle.isEmpty() )
       filmtitle = i18n( "untitled" );

    path.append( QString("/%1").arg( filmtitle ) );

    QDir d( path );
    if ( !d.exists() )
    {
       d.mkdir( path );
    }

    QString film = QString("\"%1/%2_`%3 +%a_%d.%b.%G_%H%M%S`.avi\"")
                      .arg( path )
                      .arg( m_station->text() )
                      .arg( Settings::date() );

    cmd = "";

   /************************************************
    ************************************************
    **** optional use of aumix
    ************************************************
    ************************************************/
    if ( !( aumixdev.isEmpty() ) ) {
       cmd.append(QString("%1 -%2 0 -%3 R -i %4 && ")
                 .arg(aumix)
                 .arg( aumixdev )
                 .arg( aumixdev )
                 .arg( Settings::igain_volume() ) );
    }


   /************************************************
    ************************************************
    **** optional use of nice
    ************************************************
    ************************************************/
    if (nice_value > 0)  {
       cmd.append(QString("%1 %2 -%3 ")
                 .arg( Settings::sudo() )
                 .arg( Settings::nice() )
                 .arg( nice_value ) );
    }
    cmd.append(Settings::mencoder());
    // Hardwaresettings
    cmd.append(" tv:// ");
    cmd.append(QString("-tv driver=%1:norm=%2:device=%3:input=%4")
                  .arg(Settings::driver())
                  .arg(Settings::norm())
                  .arg(Settings::videodev())
                  .arg(Settings::input()));

    // these are rather hardwareoptions as they are used for cards with ugly defaults
    int brightness = Settings::brightness();
    if ( brightness != 0 ) {
       cmd.append(QString(":brightness=%1").arg(brightness));
    }
    int hue = Settings::hue();
    if ( hue  != 0 ) {
       cmd.append(QString(":hue=%1").arg(hue));
    }
    int saturation = Settings::saturation();
    if ( saturation != 0 )  {
       cmd.append(QString(":saturation=%1").arg(saturation));
    }
    int contrast = Settings::contrast();
    if ( contrast != 0 ) {
       cmd.append(QString(":contrast=%1").arg(contrast));
    }

    // qualityoptions
    //cmd.append(":outfmt=yuy2");
    cmd.append(":quality=0");
    cmd.append(":width="  + x_scale +
               ":height=" + y_scale);

    if (useFrq == true )  {
       QString freq = m_frq->text();
       if ( freq.isEmpty() )  {
          KMessageBox::error (
               this,
               i18n ("Frequency is not set") ,
               i18n ("Command not build") );
          return QString::null;
       }
       cmd.append(":freq=" + freq );
    }  else  {
       QString channel = m_channel->text();
       if ( channel.isEmpty() )  {
          KMessageBox::error (
               this,
               i18n ("Channel is not set") ,
               i18n ("Command not build") );
          return QString::null;
       }
       cmd.append(":channel=" + channel );
    }

   /************************************************
    * free editable en/decoder options
    ************************************************/
    QString postcodingpars = Settings::postcodingpars();
    if ( ! postcodingpars.isEmpty() ) {
       cmd.append(":");
       cmd.append(postcodingpars);
    }

   /************************************************
    * lavc options
    ************************************************/
///@TODO maybe sometime I will make this optional (see also under RecordCmdDVB)
    cmd.append(" -ovc lavc -lavcopts vcodec=mpeg4");

    double bitrate  = setBitrate();
    if ( bitrate < 0 )
       return QString::null;
    cmd.append(QString(":vbitrate=%1").arg(bitrate));

    int vratetol = Settings::vratetol();
    if ( vratetol > 0 ) {
       cmd.append(QString(":vratetol=%1").arg(vratetol));
    }

    int mbd = Settings::mbd();
    if ( mbd > 0 ) {
       cmd.append(QString(":mbd=%1").arg(mbd));
    }

    int precmp = Settings::precmp();
    if ( precmp > 0 ) {
       cmd.append(QString(":precmp=%1").arg(precmp));
    }

    int cmp = Settings::cmp();
    if ( cmp > 0 ) {
       cmd.append(QString(":cmp=%1").arg(cmp));
    }

    int subcmp = Settings::subcmp();
    if ( subcmp > 0 ) {
       cmd.append(QString(":subcmp=%1").arg(subcmp));
    }

    QString aspect = Settings::aspect();
    if (! aspect.isEmpty()) {
       cmd.append(QString(":aspect=%1").arg(aspect));
    }

    if ( Settings::use_keyint() == true )  {
       cmd.append(":keyints=" + Settings::keyint() );
    }

    if (Settings::grey() == true)  {
       cmd.append(":gray");
    }

   /************************************************
    * free editable en/decoder options
    ************************************************/
    QString postlavcopts = Settings::postlavcopts();
    if ( ! postlavcopts.isEmpty() ) {
       cmd.append(":");
       cmd.append(postlavcopts);
    }

   /************************************************
    * mp3 audio options
    ************************************************/
    cmd.append(" -oac mp3lame -lameopts ");
    cmd.append("abr:br=128:mode=0");

   /************************************************
    ************************************************
    ****  F i l t e r s
    ************************************************
    ************************************************/

    QString filter = QString::null;

   /************************************************
    * crop
    ************************************************/
    QString crop = Settings::crop();
    if (! crop.isEmpty()) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append("crop=" + crop );
    }
   /************************************************
    * free editable filter options
    ************************************************/
    QString prefilter = Settings::prefilter();
    if ( ! prefilter.isEmpty() ) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append(prefilter);
    }
   /************************************************
    * deinterlace, free editable pp options
    ************************************************/
    QString pp = Settings::pp();
    if ( ! pp.isEmpty() ) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append("pp=");
       filter.append(pp);
    }
   /************************************************
    * scale
    ************************************************/
    QString filterScale = Settings::filter_scale();
    if ((! (filterScale.isEmpty())) and 
        (filterScale != QString::null) and
        (filterScale.contains("\[") <= 0 ))  {
        filterScale    = filterScale.section( " ", 0, 0 );
        if (! filter.isEmpty()) {
           filter.append(",");
        }
        filter.append("scale=" + filterScale);
    }

   /************************************************
    * denoiser
    ************************************************/
    QString denoiser = Settings::denoiser();
    if ( (! denoiser.isEmpty() ) and ( denoiser != "[none]" ) ) {
        if (! filter.isEmpty()) {
           filter.append(",");
        }
       filter.append( denoiser + "=" );
        filter.append( QString("%1:%2:%3")
                            .arg( Settings::denoise3d_chroma() )
                            .arg( Settings::denoise3d_luma() )
                            .arg( Settings::denoise3d_time() )
                            );
    }
   /************************************************
    * free editable filter options
    ************************************************/
    QString postfilter = Settings::postfilter();
    if ( ! postfilter.isEmpty() ) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append(postfilter);
    }

    if (! filter.isEmpty()) {
       cmd.append(" -vf " + filter );
    }

   /************************************************
    ************************************************
    ****  E n d    o f     F i l t e r s
    ************************************************
    ************************************************/

   /************************************************
    ************************************************
    **** other options
    ************************************************
    ************************************************/

   /* Mencoder >= 1.0pre7 uses it's own fourcc (FMP4) which makes
      it incompatible with many dvd/divx players. Restore the old
      behaviour by setting it to "DIVX". */
    cmd.append(" -ffourcc DIVX");

    cmd.append(" -endpos " + m_duration->time().toString());
    cmd.append(" -o " + film);

   /************************************************
    ************************************************
    **** verbosity level
    ************************************************
    ************************************************/
    int verbositylevel = Settings::verbosity_level();
    for (int i = 1; i <= verbositylevel; i++ )
    {
       cmd.append(" -v ");
    }

   /************************************************
    ************************************************
    **** optional use of aumix
    ************************************************
    ************************************************/
    if ( !( aumixdev.isEmpty() ) ) {
       cmd.append(QString(" && %1 -%2 %3 -%4 P -i 0 ")
                 .arg(aumix)
                 .arg( aumixdev )
                 .arg( Settings::audiodev_volume() )
                 .arg( aumixdev ) );
    }
    return cmd;
}

// BUILD the mencoder call
///@EXPERIMENTAL
QString KalvaRecorder::recordCmdDVB()
{
///@QUESTION Does this make sense? possible with DVB?
    tv_scale     = Settings::tv_scale();
    x_scale      = tv_scale.section( ":", 0, 0 );
    y_scale      = tv_scale.section( ":", 1, 1 );
    y_scale      = y_scale.section( " ", 0, 0 );

    int nice_value    = Settings::nice_value();

    QString path      = Settings::storage_path();
    QString filmtitle = m_film->currentText();
    if ( filmtitle.isEmpty() )
       filmtitle = i18n( "untitled" );

    path.append( QString("/%1").arg( filmtitle ) );

    QDir d( path );
    if ( !d.exists() )
    {
       d.mkdir( path );
    }

    QString film = QString("\"%1/%2_`%3 +%a_%d.%b.%G_%H%M%S`.avi\"")
                      .arg( path )
                      .arg( m_station->text() )
                      .arg( Settings::date() );

    cmd = "";

   /************************************************
    ************************************************
    **** optional use of nice
    ************************************************
    ************************************************/
    if (nice_value > 0)  {
       cmd.append(QString("%1 %2 -%3 ")
                 .arg( Settings::sudo() )
                 .arg( Settings::nice() )
                 .arg( nice_value ) );
    }
    cmd.append(Settings::mencoder());

    QString station = m_station->text();
    cmd.append(QString(" dvb://%1 ").arg( station ) );

    // these are rather hardwareoptions as they are used for cards with ugly defaults
///@QUESTION Does this make sense? possible with DVB?
    int brightness = Settings::brightness();
    if ( brightness != 0 ) {
       cmd.append(QString(":brightness=%1").arg(brightness));
    }
    int hue = Settings::hue();
    if ( hue  != 0 ) {
       cmd.append(QString(":hue=%1").arg(hue));
    }
    int saturation = Settings::saturation();
    if ( saturation != 0 )  {
       cmd.append(QString(":saturation=%1").arg(saturation));
    }
    int contrast = Settings::contrast();
    if ( contrast != 0 ) {
       cmd.append(QString(":contrast=%1").arg(contrast));
    }

    // qualityoptions
    //cmd.append(":outfmt=yuy2");
///@QUESTION Does this make sense? possible with DVB?
//     cmd.append(":quality=0");
//     cmd.append(":width="  + x_scale +
//                ":height=" + y_scale);

   /************************************************
    * free editable en/decoder options
    ************************************************/
    QString postcodingpars = Settings::postcodingpars();
    if ( ! postcodingpars.isEmpty() ) {
       cmd.append(":");
       cmd.append(postcodingpars);
    }

   /************************************************
    * 
    ************************************************/
    cmd.append(" -ovc copy");
///@TODO maybe sometime I will make this optional
//    cmd.append(" -ovc lavc -lavcopts vcodec=mpeg4");

//     double bitrate  = setBitrate();
//     if ( bitrate < 0 )
//        return QString::null;
//     cmd.append(QString(":vbitrate=%1").arg(bitrate));

//     int vratetol = Settings::vratetol();
//     if ( vratetol > 0 ) {
//        cmd.append(QString(":vratetol=%1").arg(vratetol));
//     }
// 
//     int mbd = Settings::mbd();
//     if ( mbd > 0 ) {
//        cmd.append(QString(":mbd=%1").arg(mbd));
//     }
// 
//     int precmp = Settings::precmp();
//     if ( precmp > 0 ) {
//        cmd.append(QString(":precmp=%1").arg(precmp));
//     }
// 
//     int cmp = Settings::cmp();
//     if ( cmp > 0 ) {
//        cmd.append(QString(":cmp=%1").arg(cmp));
//     }
// 
//     int subcmp = Settings::subcmp();
//     if ( subcmp > 0 ) {
//        cmd.append(QString(":subcmp=%1").arg(subcmp));
//     }
// 
//     QString aspect = Settings::aspect();
//     if (! aspect.isEmpty()) {
//        cmd.append(QString(":aspect=%1").arg(aspect));
//     }
// 
//     if ( Settings::use_keyint() == true )  {
//        cmd.append(":keyints=" + Settings::keyint() );
//     }
// 
//     if (Settings::grey() == true)  {
//        cmd.append(":gray");
//     }
// 
//    /************************************************
//     * free editable en/decoder options
//     ************************************************/
//     QString postlavcopts = Settings::postlavcopts();
//     if ( ! postlavcopts.isEmpty() ) {
//        cmd.append(":");
//        cmd.append(postlavcopts);
//     }

   /************************************************
    * mp3 audio options
    ************************************************/
     cmd.append(" -oac copy ");
//     cmd.append(" -oac mp3lame -lameopts ");
//     cmd.append("abr:br=128:mode=0");

   /************************************************
    ************************************************
    ****  F i l t e r s
    ************************************************
    ************************************************/

    QString filter = QString::null;

   /************************************************
    * crop
    ************************************************/
    QString crop = Settings::crop();
    if (! crop.isEmpty()) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append("crop=" + crop );
    }
   /************************************************
    * free editable filter options
    ************************************************/
    QString prefilter = Settings::prefilter();
    if ( ! prefilter.isEmpty() ) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append(prefilter);
    }
   /************************************************
    * deinterlace, free editable pp options
    ************************************************/
//     QString pp = Settings::pp();
//     if ( ! pp.isEmpty() ) {
//        if (! filter.isEmpty()) {
//           filter.append(",");
//        }
//        filter.append("pp=");
//        filter.append(pp);
//     }
   /************************************************
    * scale
    ************************************************/
//     QString filterScale = Settings::filter_scale();
//     if ((! (filterScale.isEmpty())) and 
//         (filterScale != QString::null) and
//         (filterScale.contains("\[") <= 0 ))  {
//         filterScale    = filterScale.section( " ", 0, 0 );
//         if (! filter.isEmpty()) {
//            filter.append(",");
//         }
//         filter.append("scale=" + filterScale);
//     }

   /************************************************
    * denoiser
    ************************************************/
//     QString denoiser = Settings::denoiser();
//     if ( (! denoiser.isEmpty() ) and
//          ( denoiser != QString::null ) and
//          ( denoiser.contains("\[") <= 0 ) )  {
//         if (! filter.isEmpty() )
//            filter.append(",");
//         filter.append( denoiser + "=" );
//         filter.append( QString("%1:%2:%3")
//                             .arg( Settings::denoise3d_chroma() )
//                             .arg( Settings::denoise3d_luma() )
//                             .arg( Settings::denoise3d_time() )
//                             );
//     }
   /************************************************
    * free editable filter options
    ************************************************/
    QString postfilter = Settings::postfilter();
    if ( ! postfilter.isEmpty() ) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append(postfilter);
    }

    if (! filter.isEmpty()) {
       cmd.append(" -vf " + filter );
    }

   /************************************************
    ************************************************
    ****  E n d    o f     F i l t e r s
    ************************************************
    ************************************************/

   /************************************************
    ************************************************
    **** other options
    ************************************************
    ************************************************/

   /* Mencoder >= 1.0pre7 uses it's own fourcc (FMP4) which makes
      it incompatible with many dvd/divx players. Restore the old
      behaviour by setting it to "DIVX". */
//     cmd.append(" -ffourcc DIVX");

    cmd.append(" -frames $((25*60*90))" ); 

    cmd.append(" -endpos " + m_duration->time().toString());
    cmd.append(" -o " + film);

   /************************************************
    ************************************************
    **** verbosity level
    ************************************************
    ************************************************/
    int verbositylevel = Settings::verbosity_level();
    for (int i = 1; i <= verbositylevel; i++ )
    {
       cmd.append(" -v ");
    }

    return cmd;
}

double KalvaRecorder::setBitrate()
{
   double bitrate        = Settings::bitrate();
   double maxBitrate     = Settings::maxBitrate();

   // compute duration in seconds
   int hours   = m_duration->time().hour();
   int minutes = m_duration->time().minute();
   int seconds = m_duration->time().second();
   minutes += (hours * 60);
   if ((minutes > 0) and (seconds > 0))
      minutes -= 1;
   seconds += ( minutes * 60 );

   // test duration
   if ( seconds <= 0 )  {
      KMessageBox::error (
           this,
           i18n ("Duration is not set") ,
           i18n ("Command not build") );
      return -1;
   }

   // if the hardware profile defines a maximum bitrate
   // the bitrate from the quality profile must not be
   // greater
   if ( maxBitrate && ( maxBitrate < bitrate ) )
       bitrate = maxBitrate;

   // Only test resulting filesize when the user wants this
   if ( ! Settings::testBitrate() )
       return bitrate;

   // compute the maximum bitrate that is allowed if the filesize should
   // stay under 2GB
   double reducedBitrate = std::floor((( 1995 - 0.9375 * minutes) *
                                 8192/( seconds )));

   // test if bitrate is exceeding the computed bitrate
   if ( reducedBitrate < bitrate )  {
      int ret = KMessageBox::Continue;
      if ( Settings::bitrateWarnOnly() )  {
         ret = KMessageBox::warningContinueCancel ( this,
                  i18n("<qt>Using the bitrate <b>%1</b> would result in a <i>filesize &gt;= 2GB</i>. Shall I <i>reduce</i> the bitrate to <b>%2</b> to keep the filesize &lt; 2GB ?</qt>")
                      .arg(bitrate)
                      .arg(reducedBitrate), 
                  i18n("Reduce bitrate?") );
      }
      if ( ret == KMessageBox::Continue )
         bitrate = reducedBitrate;
   }
   return bitrate;
}

QString KalvaRecorder::recordNow(
                    const QString & yourTitel,
                    const QString & yourLength,
                    bool  run
                    )
{
   m_recordMode->setCurrentPage(0);
   m_film->setCurrentText( yourTitel );
   m_duration->setTime( QTime::fromString ( yourLength ) );
   if ( run )  {
     slotRecordNow();
   }
   return i18n("recording immediatly");
}

QString KalvaRecorder::recordMovie(
                    const QString & yourTitel,
                    const QString & yourLength,
                    const QString & yourStartTime,
                    const QString & yourStartDay,
                    const QString & jobId,
                    bool  run
                    )
{
   m_jobId->setText( jobId );
   m_recordMode->setCurrentPage(1);
   m_film->setCurrentText( yourTitel );
   m_duration->setTime( QTime::fromString ( yourLength ) );
   m_startMovie->setTime( QTime::fromString ( yourStartTime ) );

   ///@TODO make this functional!???
   m_calender->setDate( QDate::fromString ( yourStartDay, Qt::ISODate ) );
   if ( run )  {
     slotRecordMovie();
   }
   return i18n("recording Movie");
   //return QString("ERROR:%1").arg(i18n("So sorry, not implemented yet!"));
}

QString KalvaRecorder::recordSerial(
                    const QString & yourTitel,
                    const QString & yourLength,
                    const QString & yourStartTime,
                    const QString & yourDOWs,
                    bool  run
                    )
{
   ///@TODO find out how we can put the recorder itself on top
   m_recordMode->setCurrentPage(2);
   m_film->setCurrentText( yourTitel );
   m_duration->setTime( QTime::fromString ( yourLength ) );
   m_startSerial->setTime( QTime::fromString ( yourStartTime ) );
   setYourDOWs( yourDOWs );
   if ( run )  {
      slotRecordSerial();
   }
   return i18n("recording serial");
}

#include "kalva_recorder.moc"
